/*! \file sock.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief  Contains usefull stuff for nonblocking UNIX sockets.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef SOCK_H
#define SOCK_H

#include <string>
#include <list>
#include <sys/poll.h>
#include <sys/un.h>
#include "error_base.h"

/*!\brief Basic stuff to handle nunblocking UNIX sockets.
 
 \todo All the exception documentation.
*/
namespace sock{

/*!\brief Basic nonblocking UNIX socket support (opening/closing)
 
 This class can just "open" and close a UNIX socket or a connection to
 a UNIX socket. It abstracts all the unix specific stuff.
*/
class Socket_basic{
 public:
  //! Closes the socket, if this instance did open it itself.
  ~Socket_basic(){Close();}
  /*! \warning The Copy constructor is not implemented. */
  Socket_basic(const Socket_basic& s); 
  /*!\brief Construct an empty descriptor or copy a descriptor from
   a file decriptor.
   
   Usually you construct a empty object and open a socket with the
   Listen(const std::string&), Connect(const std::string&) or 
   Accept(int). But if you have for some reason an open file descriptor, 
   you can just attach it to this class. Indeed, you can not do anything with
   it in this class, since you will not open anything and Close() will not
   close it, since you opened, neither will ~Socket_basic(). But it provides an
   general interfase.
   \param desc file descriptor (-1 to construct empty object, this is default)
  */
  Socket_basic(int desc=-1):iFD(desc),bMy(false){}; 

  /*!\brief Open and create a socket UNIX socket and listen to it.

   Listening means, you waiting for client connections. If a clients connects,
   you may create a new instance of \a Socket_basic and accept it with a call
   like \a client_con.Accept(server_sock).  Usually you need this only, if
   writing a server.
   \param file_name The file name of the socket. It must not exist and will
     be created.
  */
  void Listen(const std::string& file_name);
  /*!\brief Connect to a UNIX socket

   This is the method to connect to a server. If the server is accepting the
   connection, you are ready to communicate with \a write() and \a read().
   \param file_name The file name of the socket. It must exist and a server 
    shuold listen to it.
  */
  void Connect(const std::string& file_name);
  /*!\brief Get a connection via accepting it form a listening socket.
   
   Use this to get an file descriptor to a incomming connection on a 
   listening socket \a server_sock. You may create such a socket with the
   Listen(const std::string&) command.
   \param server_sock File descriptor whith a connection waiting for accept
  */
  void Accept(int server_sock);
  //! Close the socket, if this instance did open it itself.
  void Close();

  /*!\brief Get the file descriptor and make the instance empty.

   In some rare cases (e.g. forking), you do not want, that the destruction
   of the instance closes the socket. This is when you will use this method.
  */
  int GetFD(){bMy=false;int i=iFD;iFD=-1;return i;}
  /*!\brief Get the file descriptor

   You need the file descriptor for all basic i/o operation, e.g. \a read() or
   \a write().
  */
  operator int()const{return iFD;}
  //! Check if the socket is connected/you are listening.
  bool IsOpen()const{return iFD>=0;}

 private:
  void PrepareOpen(const std::string&,sockaddr_un&);
  int iFD; //-1: not open
  bool bMy;
};

class PollClass{
 public:
  PollClass(const PollClass&); //no impl
  PollClass():pPoll(NULL),iS(0),iF(0){}
  ~PollClass(){delete[] pPoll;}

  void AddSocket(int,short);
  void Change(int i,short e){pPoll[i].events=e;pPoll[i].revents=0;}
  void DelLastSocket(){--iF;}
  void Clear(){iF=0;}

  int Run(int i=0,int j=-1)const{return poll(pPoll+((j<0)?0:j),(j<0)?iF:1,i);}
  bool Test(size_t i,short e)const{return ((i<iF)?(pPoll[i].revents&e):false);}
  size_t size()const{return iF;}
 private:
  pollfd* pPoll;
  size_t iS,iF;
};

class Input{
 public:
  Input(const Socket_basic& s,int iS,char cs='\n'):sbSo(s),cSep(cs),iSize(iS),
   cpB(new char[iS+1]),i1st(0),iLst(0),b2Long(false){};
  ~Input(){delete[] cpB;}
  void Read();
  size_t Free()const{return iSize-1-(iLst-i1st+iSize)%iSize;}
  size_t Lines()const{return qNL.size();}
  std::pair<std::string,bool> GetLine();
 private:
  const Socket_basic& sbSo;
  const char cSep;
  const size_t iSize;
  char* cpB;
  size_t i1st,iLst;
  bool b2Long;
  std::list<size_t> qNL;
};

class Output{
 public:
  Output(const Output&);
  Output(const Socket_basic& s,size_t iS,char cs='\n'):
   sbSo(s),cSep(cs),iSize(iS){};
  virtual ~Output(){};

  void Write();
  void Add(const std::string& s){slOut.push_back(s);Write();}
  bool Empty()const{return slOut.empty();}
  bool Full()const{return slOut.size()>iSize;}
  
 private:
  const Socket_basic& sbSo;
  const char cSep;
  const size_t  iSize;
  std::list<std::string> slOut;
};
}

namespace Error{
/*!\brief An error within \a sock occured

 This covers all the situations, where opening the socket failed, an i/o error
 occured ....
 \todo Should only be thrown on socket specific errors.
*/
struct Sock:Runtime{
 virtual ~Sock(){};
 /*!\brief Construct an socket error.
  \param fu_name Code position of error (ususally \a METH_NAME).
  \param desc Error description.
 */ 
 Sock(const std::string& fu_name,const std::string& desc):
  Runtime(fu_name,desc){};
 //! Get an pretty error string.
 virtual operator std::string(){return std::string("socket error")+
#ifdef DEBUG
  " in "+sFunc+
#endif
  ": "+sWhat;}
};
}
#endif
