/*! \file thread_util.h
    \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
    \brief Threadsafe tools and tools usefull if working with threads.
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef THREAD_UTILS_H
#define THREAD_UTILS_H

#include <map>
#include "thread.h"

namespace thread{

/*!\brief Manage a list of callbacks.
 
 This class makes the use of C++ like callbacks easy. You can simply add
 or remove an instance to/from callback list. If you call the method
 Run(const T&) all callbacks get called.
 It is safe and usefull, to make a instance of this class to public member
 of the object, that uses callbacks.
 \param T The callback get a constant reference to this type as parameter.
*/
template <typename T>
class CallBack{
 public:
  CallBack(const CallBack&); /*!<\warning copy constructor not implemented*/
  CallBack():iID_C(0){};

  struct CBFunct; //!<\sa CallBack<T>::CBFunct

  /*!\brief Add a instance to callback list.

   Since CBFunct is abstract, you have to inherit it from CBFunct and overload
   the CBFunct::operator()(const Arg&). You can call this function even within
   a callback of the same instance.
  
   \param callback_inst The instance you want to add to the callback list.
   \return ID of your callback. You need this to remove the instance form list.
  */
  size_t Add(CBFunct* callback_inst);
  /*!\biref Delete a instance from callback list.
   
   You can call this function even within a callback of the same instance.
   \param ID of the callback you want to delete from callback list.
   \return true if successfully deleted (i.e. the ID did exist)
  */
  bool Del(size_t ID);

  /*!\brief Run thru the callback list and call back.
   
   Simply calls all callbacks in list, but can handle callbacks, that access
   the same instance (i.e. adding or deleting somthing from callback list)
   \param t Parameter you want to pass to all the callbacks.
  */
  void Run(const T& t);

 private:
  Mutex m;

  size_t iID_C;
  typedef std::map<size_t,CBFunct*> CBMap;
  CBMap cbmCBList;
};

template <typename T>
struct CallBack<T>::CBFunct{
 virtual ~CBFunct(){};

 typedef T Arg;

 virtual void operator()(const Arg& a)=0;
};
/*!\struct CallBack<T>::CBFunct 
 \brief Abstract base class, that can be called as call back

 You need to inherit your class form this to add it to a
 CallBack instance
 \sa CallBack
*/
/*!\fn thread::CallBack<T>::CBFunct::operator()(const Arg & a)
 \brief The callback function.

 This method gets called, if you are in the callback list and
  CallBack<T>::Run(const T&) is called.
 \param a The data you get form the calling class via Callback<T>::Run(const T&)
*/

/*!\brief Threadsafe FIFO with fixed size
 
 This is just the example of the pthread documentation written as
 templated class. Only the both ends of the FIFO are thread safe, i.e.
 you can read on an other thread as you are writing, but you cannot threadsafe
 read (write) on different threads in parallel.

 \param T data type in FIFO
*/
template <typename T>
class FiFo{
 public:
  ~FiFo(); //!< Frees allocated memory, but it is _not_ fully thread safe
  FiFo(const FiFo&); /*!< \warn No implementation of the copy constructor.*/
  /*!\brief Build a FIFO of fixed size and alllocate the memory.

   The constructor is _not_ fully thread safe.
   \param size Fixed size of the FIFO
  */
  FiFo(size_t size); 

  /*!\brief Returns the pointer to the actual read position, but does not
     mark it as read.

    You can only use the returned pointer between the \a Get_Start(bool) and 
    \a Get_End(bool) calls, since only for this time the actual entry is locked.
    You should not call this within an other \a Get_Start(bool) ...
    \a Get_End(bool) block.
   
    \param block true: Wait until some input on the write end of the FIFO
                 false: do not wait
    \return  Pointer to actual FIFO entry. \c NULL if nonlocking waiting and
      no input on read end.
  */
  const T* Get_Start(bool block);

  /*!\brief Finish the reading process started with \a Get_Start(bool)

   This makes the pointer returned by \a Get_Start(bool) invalid and you can
   decide to either put back the entry to queue or remove it form queue.

   \param _remove true: remove the acutal entry from queue, false: do not remove
           it.
  */
  void Get_End(bool _remove);

   /*!\brief Returns the pointer to the actual write position, but does not
     mark it as written.

    You can only use the returned pointer between the \a Put_Start(bool) and 
    \a Put_End(bool) calls, since only for this time the actual entry is locked.
    You should not call this within an other \a Put_Start(bool) ...
    \a Put_End(bool) block.
   
    \param block true: Wait until there is space to write at the FIFO
                 false: do not wait
    \return  Pointer to actual FIFO entry. \c NULL if nonlocking waiting and
      the FIFO is full.
  */
  T* Put_Start(bool block);
   /*!\brief Finish the writing process started with \a Put_Start(bool)

   This makes the pointer returned by \a Put_Start(bool) invalid and you can
   decide to either write or discard the entry.

   \param _remove true: write the actual entry, false: discard it
  */
  void Put_End(bool _remove);
 private:
  T* tpBuf;
  const size_t iSize;
  size_t iR,iW;
  Semaphore sR,sW;
};

}
#include "thread_util.tcc"

#endif
