/*! \file vtql_utils.h
 \author Matthias Kleinmann <Matthias dot Kleinmann at gmx dot de>
 \brief Non class stuff and utils for VTQL
*/
//homepage      http://www.rzuser.uni-heidelberg.de/~mkleinma/VTQL/
//Copyright (C) 2000-2002 Matthias Kleinmann <Matthias.Kleinmann@gmx.de>

#ifndef VTQL_UTILS_H
#define VTQL_UTILS_H

#include <string>
#include <stddef.h>
#ifdef __HAVE_CONFIG_H__
#include "../config.h"
#endif

namespace VTQL{

/*!\brief The different connection modes */
enum UserMode{UM_CONFIG=0, //!< configuration mode
              UM_QUERY,    //!< query mode
              UM_EVENT,    //!< event mode
              UM_NOBODY    //!< mode nobody
             };
/*!\brief The strings to a \a UserMode

 E.g cppUserModes[UM_NOBODY]="NOBODY". The last field is \c NULL.
*/
extern const char* cppUserModes[];

/*!\brief The different page output modes */
enum ShowMode{SM_LIST=0, //!< list mode
              SM_HEX,    //!< hex mode
              SM_TEXT    //!< text mode
             };
/*!\brief The strings to a \a ShowMode

 E.g cppShow[SM_HEX]="HEX". The last field is \c NULL.
*/
extern const char* cppShowModes[];

/*!\brief Escape strings as requiered for VTQL
  
  Simply puts "'"s around the input string and escapes any "'" within it.
  \param in input string
  \return escaped string
*/
std::string EscapeString(std::string in);

/*!\brief Fetch next field while handling VTQL escaped strings
 
 Most of the VTQL output are tables, where the fields are sperated by some
 character. This function deletes the next field from \a line and returns it. It
 knows about escaped strings, so that a seperator within a escaped field is
 handeld correctly and the "'"s are removed.

 \param sep Field sperator (should not be '\'' :-).
 \param line input string (gets changed!)
 \return Next field (unescaped).
*/
std::string NextSep(char sep,std::string& line);

/*!\brief Width of a Page in characters (40)*/
extern const unsigned VT_WIDTH;
/*!\brief Height of a Page, i.e. the number of lines  (25)*/
extern const unsigned VT_HEIGHT;

/*!\brief Part of a command
 
 We make a VTQL command out of keywords followed by a number if arguments.
 E.g "STATION 123 NAME 'name'" is build if {"STATION",1},{"NAME",1}.
*/
struct Command{
  const char* sWord; //!< The keyword
  int iArgs;         //!< Number of arguments for this keyword
};

/*!\brief Array of all commands
 
 This is an array of arrays of \a Command structure. Every array entry
 describes one command. The last array entry is a empty command.
 A command is a array of the \a Command structure, where the last field
 is {"",-1}. Anything is uppercase.

 The array if filled with the contents of the file \c ../vtql.syntax.spec 
*/
extern const struct Command* cmAllCmds[];

/*!\brief Convert a character to uppercase.

  Also converts German Umlaute.

  \param c input character
  \return uppercase character
*/
inline char xtoupper(char c){
#ifdef DEBUG
 if('a'<=c && c<='z')return (c-('a'-'A'));
#endif
 switch(c){
#ifndef DEBUG
  case 'a'...'z':return(c-('a'-'A'));
#endif
  case '':return '';
  case '':return '';
  case '':return '';
  default:return c;
 };
}

/*!\brief Search for needle within a haystack as \a strstr() does,
 but case insensitive

  The needle has to be in uppercase, to keep it fast. Anyway, it is quite
  slow, since it does not use assembler and maybe I do not use a fast algorithm.
  To compare case insensitive, we use \a xtoupper(char).
  \param needle In uppercase, done with \a xtoupper(char)
  \return Position in heystack of first match, NULL if no match.
*/
const char* xstrcasestr(const char* haystack,const char* needle);
}

#endif
